"""A lightweight YouTube data scraper."""

__description__ = "A lightweight YouTube data scraper."
__url__ = "https://github.com/anekobtw/youthon"

__title__ = "youthon"
__author__ = "anekobtw"
__license__ = "MIT"
__version__ = "0.2.0"

from channel import Channel
from video import Video

__all__ = ("Channel", "Video")



import json
import re
import urllib.request
from datetime import datetime

from bs4 import BeautifulSoup


class Video:
    def __init__(self, url: str) -> None:
        response = urllib.request.urlopen(url).read().decode("utf8")
        soup = BeautifulSoup(response, "html.parser")
        videoDetails = self.get_initial_data(script_content=str(soup.find_all("script")[20]))["videoDetails"]

        self.title: str = self._get_meta_content(soup, property="og:title")
        self.video_url: str = self._get_meta_content(soup, property="og:url")
        self.thumbnail_url: str = self._get_meta_content(soup, property="og:image")
        self.views: str = int(self._get_meta_content(soup, itemprop="interactionCount"))
        self.genre: str = self._get_meta_content(soup, itemprop="genre")

        self.is_private: bool = videoDetails["isPrivate"]
        self.isLiveContent: bool = videoDetails["isLiveContent"]
        self.description: str = videoDetails["shortDescription"]
        self.author: str = videoDetails["author"]
        self.legth_seconds: int = int(videoDetails["lengthSeconds"])

        print(type(self.isLiveContent))
        self.date_published: datetime = datetime.fromisoformat(soup.find("meta", itemprop="datePublished")["content"])

    @staticmethod
    def _get_meta_content(soup: BeautifulSoup, property: str = None, itemprop: str = None) -> str:
        try:
            if property:
                content = soup.find("meta", property=property)["content"]
            elif itemprop:
                content = soup.find("meta", itemprop=itemprop)["content"]
            return str(content)
        except (TypeError, KeyError, ValueError):
            return ""

    @staticmethod
    def get_initial_data(script_content: str) -> str | Exception:
        pattern = re.compile(r"var ytInitialPlayerResponse = ({.*?});", re.DOTALL)
        match = pattern.search(script_content)

        if match:
            json_str = match.group(1)
            return json.loads(json_str)
        else:
            raise Exception("ytInitialPlayerResponse not found.")



import channel
import video

channel = channel.Channel("https://www.youtube.com/@PewDiePie")

print(channel.name)  # PewDiePie
print(channel.description)  # I make videos.
print(channel.channel_url)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw
print(channel.profile_photo_url)  # https://yt3.googleusercontent.com/5oUY3tashyxfqsjO5SGhjT4dus8FkN9CsAHwXWISFrdPYii1FudD4ICtLfuCw6-THJsJbgoY=s900-c-k-c0x00ffffff-no-rj
print(channel.channel_id)  # UC-lHJZR3Gqxm24_Vd_AJ5Yw

# The following two variables may produce different results depending on your geolocation.
print(channel.subscribers_count)
print(channel.video_count)

print(channel.videos_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/videos
print(channel.shorts_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/shorts
print(channel.community_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/community
print(channel.featured_channels_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/channels
print(channel.playlists_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/playlists
print(channel.about_page)  # https://www.youtube.com/channel/UC-lHJZR3Gqxm24_Vd_AJ5Yw/about


video = video.Video("https://youtu.be/XqZsoesa55w")

print(video.title)  #  Baby Shark Dance | #babyshark Most Viewed Video | Animal Songs | PINKFONG Songs for Children
print(video.description)  # it returns a looong description
print(video.author)  # Baby Shark - Pinkfong Kids’ Songs & Stories
print(video.video_url)  # https://www.youtube.com/watch?v=XqZsoesa55w

print(video.views)  # 14941899516
print(video.thumbnail_url)  #https://i.ytimg.com/vi/XqZsoesa55w/maxresdefault.jpg
print(video.date_published)  # 2016-06-17 16:00:30-07:00
print(video.legth_seconds)  # 136
print(video.isLiveContent)  # False
print(video.is_private)  # False
print(video.genre)  # Education



import json
import re
import urllib.request

from bs4 import BeautifulSoup


class Channel:
    def __init__(self, url: str) -> None:
        response = urllib.request.urlopen(url).read().decode("utf8")
        soup = BeautifulSoup(response, "html.parser")
        initial_data = self.get_initial_data(script_content=str(soup.find_all("script")[24]))

        self.name = self._get_meta_content(soup, 5)
        self.description: str = self._get_meta_content(soup, 11)
        self.channel_url: str = self._get_meta_content(soup, 7)
        self.profile_photo_url: str = self._get_meta_content(soup, 8)

        metadataParts = initial_data["header"]["pageHeaderRenderer"]["content"]["pageHeaderViewModel"]["metadata"]["contentMetadataViewModel"]["metadataRows"][1]["metadataParts"]
        self.subscribers_count: str = metadataParts[0]["text"]["content"].split(" ")[0]
        self.video_count: str = metadataParts[1]["text"]["content"].split(" ")[0]
        self.channel_id: str = initial_data["metadata"]["channelMetadataRenderer"]["externalId"]

        self.videos_page: str = self.channel_url + "/videos"
        self.shorts_page: str = self.channel_url + "/shorts"
        self.playlists_page: str = self.channel_url + "/playlists"
        self.community_page: str = self.channel_url + "/community"
        self.featured_channels_page: str = self.channel_url + "/channels"
        self.about_page: str = self.channel_url + "/about"

    @staticmethod
    def _get_meta_content(soup: BeautifulSoup, index: int) -> str:
        try:
            return str(soup.find_all("meta")[index]["content"])
        except (IndexError, KeyError):
            return ""

    @staticmethod
    def get_initial_data(script_content: str) -> str | Exception:
        pattern = re.compile(r"var ytInitialData = ({.*?});", re.DOTALL)
        match = pattern.search(script_content)

        if match:
            json_str = match.group(1)
            return json.loads(json_str)
        else:
            raise Exception("ytInitialData not found.")
